import glob, os
import numpy as np
import pandas as pd
import re
import pytopdrawer as ptd
from pytopdrawer import TopPlot
from scipy.stats import chisquare
from scipy.stats import chi2

def load_top_plot(plot:TopPlot):
	pairs = [
		("pvalue",pvalue_top(plot)),
		("chi2",chisquare_top(plot)),
		("plot",plot),
	]
	return pd.DataFrame.from_records(pairs,columns=['title',plot.title.text],index="title").transpose()

def load_top_file(file):
	pairs={}
	for top in ptd.read(file):
		name = re.compile('(pwg[a-zA-Z0-9-]+?)(\d\d\d\d)?-([a-zA-Z0-9-]+?grid).top')
		g = name.search(file)
		if g is not None:
			fname = g.group(1)
			if fname[-1] == "-":
				fname = fname[0:-1]
			fname = fname + "-" + g.group(3)
			number = int(g.group(2))
			if number not in pairs.keys():
				pairs[number] = load_top_plot(top)
			else:
				pairs[number]=pd.concat([pairs[number],load_top_plot(top)])
	return pd.concat(pairs.values(),keys =pairs.keys())

def load_top_folder(folder): # names
	pairs={}
	for file in glob.glob(folder + "/pwg*.top"):
		name = re.compile('(pwg[a-zA-Z0-9-]+?)(\d\d\d\d)?-([a-zA-Z0-9-]+?grid).top')
		g = name.search(file)
		if g is not None:
			fname = g.group(1)
			if fname[-1] == "-":
				fname = fname[0:-1]
			fname = fname + "-" + g.group(3)
			if g.group(2) is not None:
				number = int(g.group(2))
				if fname not in pairs.keys():
					pairs[fname] = load_top_file(file)
				else:
					pairs[fname]=pd.concat([pairs[fname],load_top_file(file)])
	return pd.concat(pairs.values(),keys =pairs.keys())

	
def pvalue_top(top:TopPlot):
	return chi2.sf(chisquare_top(top),1)
def chisquare_top(top:TopPlot):
	mask = top.xdata() >0 
	return np.sum((top.ydata()[mask]- top.xdata()[mask])**2/top.xdata()[mask])

	#chi2 = chisquare(top.ydata()[mask], top.xdata()[mask])
	# return chi2




#def analyze_top(top):

def smoothness_test(folder):
	raise Exception("Not implemented")

def chisquare_tops(folder,p_min = 0.33):
	for file in glob.glob(folder + "/*.top"):
		tops = ptd.read(file)
		for top in tops:
			chi2 = chisquare_top(top)
			p  = pvalue_top(top)
			if (p < p_min):
				print("p=", p)
				top.show()

def btlgrid_tops(folder,p_min=0.95):
	for file in glob.glob(folder + "/*btlgrid.top"):
		title = re.compile('(pwg-[a-zA-Z0-9-]+)-(\d*)-?btlgrid.top')

		tops = ptd.read(file)
		for top in tops:
			mask = top.xdata() >0 
			chi2 = chisquare(top.ydata()[mask], top.xdata()[mask])
			p  = pvalue_top(top.ydata()[mask], top.xdata()[mask])
			if (p < p_min):
				print("p=", p)
				top.show()



#!/usr/bin/python3
import argparse
import pytopdrawer
import pypowhegparse as ppp
import numpy as np
import sys 
from smpl import plot


def main():
	parser = argparse.ArgumentParser()
	parser.add_argument('-f','--folder', type=str,help="url/file/string",default=".")
	parser.add_argument('-w','--warn', type=int,help="level of warnings to fail",default=3)
	parser.add_argument('-wt','--warn-threshold', type=int,help="number of warnings to fail",default=0)
	parser.add_argument('-ic','--ignore-colour', type=bool,default=False)
	parser.add_argument('-is','--ignore-spin', type=bool,default=False)

	args = parser.parse_args()
	exit_code = 0

	err = ppp.error_colour_grep(args.folder)
	if len(err)> 0 and not args.ignore_colour:
		for s in err:
			print(s)
		exit_code=1

	err = ppp.error_spin_grep(args.folder)
	if len(err)> 0 and not args.ignore_spin:
		for s in err:
			print(s)
		exit_code=1

	warn = ppp.inspect_warn_grep(args.folder,args.warn)
	if len(warn) > args.warn_threshold:
		print("Checklimits warning stats:")
		print("-"*50)
		ppp.print_stats(args.folder)
		print("-"*50)
		print()
		print("WWWARN:")
		print("-"*50)

		for a in warn:
			print()
			for s in a:
				print(s)
			print()
		print("-"*50)
	sys.exit(exit_code)

main()


import subprocess

def inspect_warn_loop(folder,level=1):
	raise Exception ("Unimplemented")

def error_colour_grep(folder):
	p = subprocess.run("grep -H colour check fails " +folder + "/*checklimits*", shell=True, capture_output=True)
	lines = p.stdout.split(b'\n')
	return lines

def error_spin_grep(folder):
	p = subprocess.run("grep -H spin correlated amplitude wrong " +folder + "/*checklimits*", shell=True, capture_output=True)
	lines = p.stdout.split(b'\n')
	return lines
	

def inspect_warn_grep(folder,level=1):
	after = 10
	before = 10
	sumd = before + after
	arg = []
	p = subprocess.run("grep -A "+str(after)+" -B "+str(before)+" -H " +"W"*level + "ARN " +folder + "/*checklimits*", shell=True, capture_output=True)
	lines = p.stdout.split(b'\n')
	return [[lines[j*sumd+i] for i in range(sumd)] for j in range(int(len(lines)/sumd))]



def search_for_warn_loop(folder,level=1):
	raise Exception ("Unimplemented")

def search_for_warn_grep(folder,level=1):
	arg = []
	p = subprocess.run("grep " +"W"*level + "ARN " +folder + "/*checklimits*", shell=True, capture_output=True)
	return p.stdout.split(b'\n')


def search_for_warn(folder,level=1,grep=True):
	if grep:
		return search_for_warn_grep(folder,level)
	else:
		return search_for_warn_loop(folder,level)

def count_warn(folder,level=1,grep=True):
	if grep:
		return len(search_for_warn_grep(folder,level))-1
	else:
		return len(search_for_warn_loop(folder,level))-1

def print_stats(folder,grep=True):
	print("#WARN    = ", count_warn(folder ,1))
	print("#WWARN   = ", count_warn(folder,2))
	print("#WWWARN  = ", count_warn(folder ,3))
	print("#WWWWARN = ", count_warn(folder ,4))

def print_warn_grep(folder,level=3):
	for a in inspect_warn_grep(folder,3):
		print()
		for s in a:
			print(s)
		print()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pypowhegparse']

package_data = \
{'': ['*']}

install_requires = \
['numpy', 'pandas', 'pytopdrawer', 'scipy', 'smpl', 'uncertainties']

entry_points = \
{'console_scripts': ['hepi-fast = pypowhegparse.pypowhegparse:main']}

setup_kwargs = {
    'name': 'pypowhegparse',
    'version': '0.0.2',
    'description': 'Analyse POWHEG output files for potential errors.',
    'long_description': '# pypowhegparse\n\nAnalyse POWHEG output files for potential errors.',
    'author': 'Alexander Puck Neuwirth',
    'author_email': 'alexander@neuwirth-informatik.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/APN-Pucky/pypowhegparse.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import re
import glob, os
import pandas as pd
from uncertainties import ufloat

def load_stat_file(file):
	pairs=[]
	name = re.compile('(pwg[a-zA-Z0-9-]+?)(\d\d\d\d)?-stat.dat')
	g = name.search(os.path.basename(file))
	if g is not None:
		fname = g.group(1)
		number = int(g.group(2))
	with open(file) as topo_file:
		for line in topo_file:
			pair = re.compile('(.*?)\s+([0-9\.Ee\+-]+)\s+\+-\s+([0-9\.Ee\+-]+)')
			g = pair.search(line)
			if g is not None:
				pairs.append((g.group(1).strip(),float(g.group(2))))
				pairs.append((g.group(1).strip() + "+-stat",float(g.group(3))))
			else:
				pair = re.compile('(.*?)\s+([0-9\.Ee\+-]+)')
				g = pair.search(line)
				if g is not None:
					pairs.append((g.group(1),float(g.group(2))))
	return pd.DataFrame.from_records(pairs,columns=['proc',number],index="proc").transpose()

def load_stat_folder(folder):
	pairs={}
	for file in glob.glob(folder + "/pwg*stat.dat"):
		name = re.compile('(pwg[a-zA-Z0-9-]+?)(\d\d\d\d)?-stat.dat')
		g = name.search(file)
		if g is not None:
			fname = g.group(1)
			if fname[-1] == "-":
				fname = fname[0:-1]
			fname = fname + "-stat"
			number = int(g.group(2))
			if fname not in pairs.keys():
				pairs[fname] = load_stat_file(file)
			else:
				pairs[fname]=pd.concat([pairs[fname],load_stat_file(file)])
	return pd.concat(pairs.values(),keys =pairs.keys())